/*========================================================================================================
 * OperatorInterface.cpp
 *========================================================================================================
 *
 * File Description:
 *
 * This implements holds onto the instances of the game controllers used by driver and the co-pilot.
 *========================================================================================================*/

#include <SmartDashboard/SmartDashboard.h>  // Smart Dashboard

// Team 302 includes
//#include <RobotMap.h>
#include <teleopControl/DragonXbox.h>
#include <teleopControl/TeleopControl.h>


//----------------------------------------------------------------------------------
// Method:      GetInstance
// Description: If there isn't an instance of this class, it will create one.  The
//              single class instance will be returned.
// Returns:     OperatorInterface*  instance of this class
//----------------------------------------------------------------------------------
TeleopControl* TeleopControl::m_instance = nullptr; // initialize the instance variable to nullptr
TeleopControl* TeleopControl::GetInstance()
{
    if ( TeleopControl::m_instance == nullptr )
    {
        TeleopControl::m_instance = new TeleopControl();
    }
    return TeleopControl::m_instance;
}
//----------------------------------------------------------------------------------
// Method:      OperatorInterface <<constructor>>
// Description: This will construct and initialize the object.
//              It maps the functions to the buttons/axis.
//---------------------------------------------------------------------------------
TeleopControl::TeleopControl() : m_axisIDs(),
								 m_buttonIDs(),
								 m_controllerIndex(),
								 m_controllers()
{
	m_controllers[0] = new DragonXbox( 0 );
	m_controllers[1] = new DragonXbox( 1 );

    // Initialize the items to not defined
    for ( int inx=0; inx<m_maxFunctions; ++inx )
    {
        m_axisIDs[inx]    		= UNDEFINED_AXIS;
        m_buttonIDs[inx]  		= UNDEFINED_BUTTON;
        m_controllerIndex[inx]  = -1;
    }

    m_controllerIndex[ TANK_DRIVE_LEFT_CONTROL ] 	= 0;
    m_axisIDs[ TANK_DRIVE_LEFT_CONTROL ]    		= XBOX_Y_AXIS_LEFT_JOYSTICK;
    m_controllerIndex[ TANK_DRIVE_RIGHT_CONTROL ] 	= 0;
    m_axisIDs[ TANK_DRIVE_RIGHT_CONTROL ]   		= XBOX_Y_AXIS_RIGHT_JOYSTICK;

    m_controllerIndex[ ARCADE_DRIVE_THROTTLE ] 		= 0;
    m_axisIDs[ ARCADE_DRIVE_THROTTLE ]      		= XBOX_Y_AXIS_LEFT_JOYSTICK;
//    m_axisIDs[ ARCADE_DRIVE_THROTTLE ]      		= XBOX_RIGHT_TRIGGER;
    m_controllerIndex[ ARCADE_DRIVE_STEER ] 		= 0;
    m_axisIDs[ ARCADE_DRIVE_STEER ]         		= XBOX_X_AXIS_RIGHT_JOYSTICK;
//    m_axisIDs[ ARCADE_DRIVE_STEER ]         		= XBOX_X_AXIS_LEFT_JOYSTICK;

    m_controllerIndex[ STRAFE_DRIVE_THROTTLE ] 		= 0;
    m_axisIDs[ STRAFE_DRIVE_THROTTLE ]				= XBOX_Y_AXIS_LEFT_JOYSTICK;
    m_controllerIndex[ STRAFE_DRIVE_STEER ] 		= 0;
    m_axisIDs[ STRAFE_DRIVE_STEER ]					= XBOX_X_AXIS_LEFT_JOYSTICK;
    m_controllerIndex[ STRAFE_ROTATE_AXIS ] 		= 0;
    m_axisIDs[ STRAFE_ROTATE_AXIS ]					= XBOX_X_AXIS_RIGHT_JOYSTICK;

    m_controllerIndex[ SWITCH_DRIVE_MODE ] 			= 0;
    m_buttonIDs[ SWITCH_DRIVE_MODE ]        		= XBOX_Y_BUTTON;

}


//------------------------------------------------------------------
// Method:      SetAxisScaleFactor
// Description: Allow the range of values to be set smaller than
//              -1.0 to 1.0.  By providing a scale factor between 0.0
//              and 1.0, the range can be made smaller.  If a value
//              outside the range is provided, then the value will
//              be set to the closest bounding value (e.g. 1.5 will
//              become 1.0)
// Returns:     void
//------------------------------------------------------------------
void TeleopControl::SetAxisScaleFactor
(
    TeleopControl::FUNCTION_IDENTIFIER  	function,      // <I> - function that will update an axis
    float                                   scaleFactor    // <I> - scale factor used to limit the range
)
{
	int ctlIndex = m_controllerIndex[ function];
	AXIS_IDENTIFIER axis = m_axisIDs[ function ];
    if ( ctlIndex > -1 && axis != UNDEFINED_AXIS  )
    {
        m_controllers[ ctlIndex ]->SetAxisScale( axis,scaleFactor);
    }
}

//------------------------------------------------------------------
// Method:      SetAxisProfile
// Description: Sets the axis profile for the specifed axis
// Returns:     void
//------------------------------------------------------------------
void TeleopControl::SetAxisProfile
(
    TeleopControl::FUNCTION_IDENTIFIER      function,       // <I> - function that will update an axis
    AXIS_SENSITIVITY_CURVE          		profile         // <I> - profile to use
)
{
	int ctlIndex = m_controllerIndex[ function];
	AXIS_IDENTIFIER axis = m_axisIDs[ function ];
    if ( ctlIndex > -1 && axis != UNDEFINED_AXIS  )
    {
    	m_controllers[ ctlIndex ]->SetAxisProfile( axis,profile);
    }
}

//------------------------------------------------------------------
// Method:      GetAxisValue
// Description: Reads the joystick axis, removes any deadband (small
//              value) and then scales as requested.
// Returns:     float   -  scaled axis value
//------------------------------------------------------------------
float TeleopControl::GetAxisValue
(
    TeleopControl::FUNCTION_IDENTIFIER  function    // <I> - function that whose axis will be read
) const
{
    float value = 0.0;
	int ctlIndex = m_controllerIndex[ function];
	AXIS_IDENTIFIER axis = m_axisIDs[ function ];
    if ( ctlIndex > -1 && axis != UNDEFINED_AXIS  )
    {
        value = m_controllers[ ctlIndex ]->GetAxisValue( axis );
    }
	SmartDashboard::PutNumber( "Axis ID ", function );
	SmartDashboard::PutNumber( "Axis Value ", value );
    return value;
}

//------------------------------------------------------------------
// Method:      GetRawButton
// Description: Reads the button value.  Also allows POV, bumpers,
//              and triggers to be treated as buttons.
// Returns:     bool   -  scaled axis value
//------------------------------------------------------------------
bool TeleopControl::IsButtonPressed
(
    TeleopControl::FUNCTION_IDENTIFIER  function    // <I> - function that whose button will be read
) const
{
    bool isSelected = false;
	int ctlIndex = m_controllerIndex[ function];
	BUTTON_IDENTIFIER btn = m_buttonIDs[ function ];
    if ( ctlIndex > -1 && btn != UNDEFINED_BUTTON  )
    {
        isSelected = m_controllers[ ctlIndex ]->IsButtonPressed( btn );
    }
    return isSelected;
}


