#include <iostream>

#include <SmartDashboard/SmartDashboard.h>

#include <thirdParty/puxixml/pugixml.hpp>

#include <subsys/xmlParsing/MotorDefn.h>
#include <subsys/components/DragonTalon.h>

//-----------------------------------------------------------------------
// Method:      ParseXML
// Description: Parse a motor XML element and create a DragonTalon from
//              its definition.
// Returns:     DragonTalon*        motor controller (or nullptr if XML
//                                  is ill-formed
//-----------------------------------------------------------------------
DragonTalon* MotorDefn::ParseXML
(
    pugi::xml_node      motorNode
)
{
    DragonTalon* talon = nullptr;

    int canID = 0;
    const char* motorName = "";
    bool        inverted = false;
    bool        sensorInverted = false;
    int         countsPerRev = 0;
    float       gearRatio = 1;
    bool        brakeMode = false;
    int         currentLimit = 0;
    bool        enableCurrentLimit = false;
    bool        enableSafety = false;
    int         masterID = -1;

    bool hasError = false;

    std::cout << "Motor Node:";
    std::cout << " " << motorNode.name();

    for (pugi::xml_attribute attr = motorNode.first_attribute(); attr; attr = attr.next_attribute())
    {

        std::cout << " " << attr.name() << "=" << attr.value();

        if ( strcmp( attr.name(), "name" ) == 0 )
        {
            motorName = attr.value();
        }
        else if ( strcmp( attr.name(), "canId" ) == 0 )
        {
            canID = attr.as_int();
        }
        else if ( strcmp( attr.name(), "type" ) == 0 )
        {
        	if ( strcmp( attr.value(), "cantalon" ) != 0 )
        	{
        		hasError = true;
        	}
        }
        else if ( strcmp( attr.name(), "inverted" ) == 0 )
        {
            inverted = attr.as_bool();
        }
        else if ( strcmp( attr.name(), "sensorInverted" ) == 0 )
        {
            sensorInverted = attr.as_bool();
        }
        else if ( strcmp( attr.name(), "countsPerRev" ) == 0 )
        {
            countsPerRev = attr.as_int();
        }
        else if ( strcmp( attr.name(), "gearRatio" ) == 0 )
        {
            gearRatio = attr.as_float();
        }

        else if ( strcmp( attr.name(), "brakeMode" ) == 0 )
        {
            brakeMode = attr.as_bool();
        }
        else if ( strcmp( attr.name(), "currentLimiting" ) == 0 )
        {
            currentLimit = attr.as_int();
            if ( currentLimit > 0 )
            {
                enableCurrentLimit = true;
            }
        }
        else if ( strcmp( attr.name(), "safety" ) == 0 )
        {
            enableSafety = attr.as_bool();
        }
        else if ( strcmp( attr.name(), "masterID" ) == 0 )
        {
            masterID = attr.as_int();
        }
        else
        {
            std::cout << " invalid attribute " << attr.name();
            hasError = true;
        }
    }

    std::cout << std::endl;

    std::cout << "Motor:  " << motorName << " ID " << canID << std::endl;
    std::cout << "inverted: " << inverted << std::endl;
    std::cout << "sensor inverted: " << sensorInverted << std::endl;
    std::cout << "counts per rev: " << countsPerRev << std::endl;
    std::cout << "gear ratio: " << gearRatio << std::endl;
    std::cout << "brake mode: " << brakeMode << std::endl;
    std::cout << "current limit: " << currentLimit << std::endl;
    std::cout << "enable current limit: " << enableCurrentLimit << std::endl;
    std::cout << "safety: " << enableSafety << std::endl;
    std::cout << "master: " << masterID << std::endl;

    if ( !hasError )
    {
        printf( "Creating DragonTalon \n");
        talon = new DragonTalon( motorName, canID );
        talon->SetInverted( inverted );
        talon->SetSensorDirection( sensorInverted );
        talon->SetFeedbackDevice( CANTalon::CtreMagEncoder_Relative ); // TODO: read from file
        printf( " sensor present %d \n", talon->IsSensorPresent( CANTalon::CtreMagEncoder_Relative ) );
        talon->ConfigEncoderCodesPerRev( countsPerRev * gearRatio );
        if ( brakeMode )
        {
            talon->ConfigNeutralMode( frc::CANSpeedController::kNeutralMode_Brake );
        }
        else
        {
            talon->ConfigNeutralMode( frc::CANSpeedController::kNeutralMode_Coast );
        }
        talon->SetCurrentLimit( currentLimit );
        talon->EnableCurrentLimit( enableCurrentLimit );
        talon->SetSafetyEnabled( enableSafety );
    }
    return talon;
}
