/*
 * CompBotChassis.h
 *
 *  Created on: Jan 16, 2017
 *      Author: Tumtu/Neethan Punniyamoorthy
 */

#ifndef SRC_SUBSYS_MECANUMCHASSIS_H_
#define SRC_SUBSYS_MECANUMCHASSIS_H_

#include <CANTalon.h>
#include <subsys/IChassis.h>

#include <subsys/RobotMap.h>


namespace Team302
{

	class MecanumChassis : public IChassis
	{
		public:

			//--------------------------------------------------------------------------------
			// Method:			IChassis
			// Description: 	Default Constructor
			// Returns: 		void
			//--------------------------------------------------------------------------------
			MecanumChassis();

			//--------------------------------------------------------------------------------
			// Method:			~IChassis
			// Description: 	Default Destructor
			// Returns: 		void
			//--------------------------------------------------------------------------------
			virtual ~MecanumChassis() = default;

			//===========================  Accessors ========================================
			//==================== Velocity
			//--------------------------------------------------------------------------------
			// Method:			GetLeftFrontVelocity
			// Description: 	Return the calculated velocity of the left front wheel
			// Returns: 		float		velocity in inches / second
			//--------------------------------------------------------------------------------
			float GetLeftFrontVelocity() const override;

			//--------------------------------------------------------------------------------
			// Method:			GetLeftBackVelocity
			// Description: 	Return the calculated velocity of the left Back wheel
			// Returns: 		float		velocity in inches / second
			//--------------------------------------------------------------------------------
			float GetLeftBackVelocity() const override;

			//--------------------------------------------------------------------------------
			// Method:			GetRightFrontVelocity
			// Description: 	Return the calculated velocity of the right front wheel
			// Returns: 		float		velocity in inches / second
			//--------------------------------------------------------------------------------
			float GetRightFrontVelocity() const override;

			//--------------------------------------------------------------------------------
			// Method:			GetRightBackVelocity
			// Description: 	Return the calculated velocity of the right Back wheel
			// Returns: 		float		velocity in inches / second
			//--------------------------------------------------------------------------------
			float GetRightBackVelocity() const override;

			//==================== Displacement
			//--------------------------------------------------------------------------------
			// Method:			GetLeftFrontDistance
			// Description: 	Return the calculated distance of the left front wheel
			// Returns: 		float		velocity in inches
			//--------------------------------------------------------------------------------
			float GetLeftFrontDistance() const override;

			//--------------------------------------------------------------------------------
			// Method:			GetLeftBackDistance
			// Description: 	Return the calculated distance of the left Back wheel
			// Returns: 		float		velocity in inches
			//--------------------------------------------------------------------------------
			float GetLeftBackDistance() const override;

			//--------------------------------------------------------------------------------
			// Method:			GetRightFrontDistance
			// Description: 	Return the calculated distance of the right front wheel
			// Returns: 		float		velocity in inches
			//--------------------------------------------------------------------------------
			float GetRightFrontDistance() const override;

			//--------------------------------------------------------------------------------
			// Method:			GetRightBackDistance
			// Description: 	Return the calculated distance of the right Back wheel
			// Returns: 		float		velocity in inches
			//--------------------------------------------------------------------------------
			float GetRightBackDistance() const override;

			//===========================  Mutators ========================================
			//--------------------------------------------------------------------------------
			// Method:			ResetDistance
			// Description: 	Reset the distance accumulator, so that the GetxxxDistance calls
			//					are relative to the current position.
			// Returns: 		void
			//--------------------------------------------------------------------------------
			void ResetDistance() override;

			//==================== Current Limiting
			//--------------------------------------------------------------------------------
			// Method:			SetCurrentLimit
			// Description: 	Set the maximum current for the drive wheels
			// Returns: 		void
			//--------------------------------------------------------------------------------
			void SetCurrentLimit
			(
				int amps			// <I> - current limit in amps for each motor
			) override;

			//--------------------------------------------------------------------------------
			// Method:			EnableCurrentLimiting
			// Description: 	Turn on/off current limiting
			// Returns: 		int			error - 0 = everything ok
			//										1 = no current limit set
			//--------------------------------------------------------------------------------
			int EnableCurrentLimiting
			(
				bool	on			// <I> - true - turn on current enabling, false - turn it off
			) override;


			//==================== Neutral Mode
			//--------------------------------------------------------------------------------
			// Method:			SetNeutralMode
			// Description: 	Set the neutral mode on the motor
			// Returns: 		void
			//--------------------------------------------------------------------------------
			void SetNeutralMode
			(
				CANSpeedController::NeutralMode		mode //	 <I> - the CANSpeedController neutral mode (usually either kNeutralMode_Brake or kNeutralMode_Coast)
			) override;

			//==================== Open Loop
			//--------------------------------------------------------------------------------
			// Method:			SetOpenLoopPower
			// Description: 	Set the power to the drive wheels (this is using a talon
			//					control mode kThrottleMode).
			//
			//					Values outside the range will be snapped to closest bounding value.
			//
			//					Due to different chassis configurations (e.g. some chassis may have
			//					wheels mechanically linked together).  Some motors will be followers
			//					or slaves to the masters.  In these cases the power to the from
			//					motors will be used and the back motors will be ignored.
			//
			// Returns: 		void
			//--------------------------------------------------------------------------------
			void SetOpenLoopPower
			(
				float	leftFrontPower,			// <I> - left front motor power ( -1.0 < value < 1.0 )
				float	leftBackPower,			// <I> - left back motor power ( -1.0 < value < 1.0 )
				float	rightFrontPower,		// <I> - right front motor power ( -1.0 < value < 1.0 )
				float	rightBackPower 			// <I> - right back motor power ( -1.0 < value < 1.0 )
			) override;

			//==================== Closed Loop
			//--------------------------------------------------------------------------------
			// Method:			SetClosedLoopParameters
			// Description: 	Set the power to the drive wheels.  Values outside the range
			//					will be snapped to closest bounding value.
			// Returns: 		void
			//--------------------------------------------------------------------------------
			void SetClosedLoopParameters
			(
				ChassisControlMode	mode,	// <I> - talon control mode
				float	proportionalConst,	// <I> - kP from a PIDF
				float	integralConst,		// <I> - kI from a PIDF
				float	derivativeConst,	// <I> - kD from a PIDF
				float	feetForwardConst,	// <I> - kF from a PIDF
				float	leftFrontTarget,	// <I> - left front motor target (units are based on the mode where distances are inches and time is in seconds)
				float	leftBackTarget,		// <I> - left back motor power (units are based on the mode where distances are inches and time is in seconds)
				float	rightFrontTarget,	// <I> - right front motor power (units are based on the mode where distances are inches and time is in seconds)
				float	rightBackTarget		// <I> - right back motor power (units are based on the mode where distances are inches and time is in seconds)
			) override;

		protected:

		private:
			CANTalon*		m_driveMotors[ RobotMap::DRIVE_MOTORS::MAX_DRIVE_MOTORS ];
			float			m_amps;

			// Default methods we don't want the compiler to implement
			MecanumChassis( const MecanumChassis& ) = delete;
			MecanumChassis& operator= ( const MecanumChassis& ) = delete;
			explicit MecanumChassis( MecanumChassis* other ) = delete;
	};

} /* namespace Team302 */

#endif /* SRC_SUBSYS_COMPBOT_COMPBOTCHASSIS_H_ */
